@@ -443,6 +443,14 @@ def hamiltonian_of_haldane_model_in_quasi_one_dimension(k, N=10, M=2/3, t1=1, t2
     hamiltonian = h00 + h01*cmath.exp(1j*k) + h01.transpose().conj()*cmath.exp(-1j*k)
     return hamiltonian
 
+def hamiltonian_of_one_QAH_model(k1, k2, t1=1, t2=1, t3=0.5, m=-1):
+    hamiltonian = np.zeros((2, 2), dtype=complex)
+    hamiltonian[0, 1] = 2*t1*cos(k1)-1j*2*t1*cos(k2)
+    hamiltonian[1, 0] = 2*t1*cos(k1)+1j*2*t1*cos(k2)
+    hamiltonian[0, 0] = m+2*t3*sin(k1)+2*t3*sin(k2)+2*t2*cos(k1+k2)
+    hamiltonian[1, 1] = -(m+2*t3*sin(k1)+2*t3*sin(k2)+2*t2*cos(k1+k2))
+    return hamiltonian
+
 
 
 
@@ -704,11 +712,13 @@ def total_density_of_states(fermi_energy, hamiltonian, broadening=0.01):
     total_dos = -np.trace(np.imag(green))/pi
     return total_dos
 
-def total_density_of_states_with_fermi_energy_array(fermi_energy_array, hamiltonian, broadening=0.01):
+def total_density_of_states_with_fermi_energy_array(fermi_energy_array, hamiltonian, broadening=0.01, print_show=0):
     dim = np.array(fermi_energy_array).shape[0]
     total_dos_array = np.zeros(dim)
     i0 = 0
     for fermi_energy in fermi_energy_array:
+        if print_show == 1:
+            print(fermi_energy)
         total_dos_array[i0] = total_density_of_states(fermi_energy, hamiltonian, broadening)
         i0 += 1
     return total_dos_array
@@ -859,12 +869,14 @@ def calculate_conductance(fermi_energy, h00, h01, length=100):
     conductance = np.trace(np.dot(np.dot(np.dot(gamma_left, green_0n_n), gamma_right), green_0n_n.transpose().conj()))
     return conductance
 
-def calculate_conductance_with_fermi_energy_array(fermi_energy_array, h00, h01, length=100):
+def calculate_conductance_with_fermi_energy_array(fermi_energy_array, h00, h01, length=100, print_show=0):
     dim = np.array(fermi_energy_array).shape[0]
     conductance_array = np.zeros(dim)
     i0 = 0
-    for fermi_energy_0 in fermi_energy_array:
-        conductance_array[i0] = np.real(calculate_conductance(fermi_energy_0, h00, h01, length))
+    for fermi_energy in fermi_energy_array:
+        if print_show == 1:
+            print(fermi_energy)
+        conductance_array[i0] = np.real(calculate_conductance(fermi_energy, h00, h01, length))
         i0 += 1
     return conductance_array
 
@@ -888,39 +900,126 @@ def calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensi
     conductance = np.trace(np.dot(np.dot(np.dot(gamma_left, green_0n_n), gamma_right), green_0n_n.transpose().conj()))
     return conductance
 
-def calculate_conductance_with_disorder_intensity_array(fermi_energy, h00, h01, disorder_intensity_array, disorder_concentration=1.0, length=100, calculation_times=1):
+def calculate_conductance_with_disorder_intensity_array(fermi_energy, h00, h01, disorder_intensity_array, disorder_concentration=1.0, length=100, calculation_times=1, print_show=0):
     dim = np.array(disorder_intensity_array).shape[0]
     conductance_array = np.zeros(dim)
     i0 = 0
-    for disorder_intensity_0 in disorder_intensity_array:
+    for disorder_intensity in disorder_intensity_array:
+        if print_show == 1:
+            print(disorder_intensity)
         for times in range(calculation_times):
-            conductance_array[i0] = conductance_array[i0]+np.real(calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensity=disorder_intensity_0, disorder_concentration=disorder_concentration, length=length))
+            conductance_array[i0] = conductance_array[i0]+np.real(calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensity=disorder_intensity, disorder_concentration=disorder_concentration, length=length))
         i0 += 1
     conductance_array = conductance_array/calculation_times
     return conductance_array
 
-def calculate_conductance_with_disorder_concentration_array(fermi_energy, h00, h01, disorder_concentration_array, disorder_intensity=2.0, length=100, calculation_times=1):
+def calculate_conductance_with_disorder_concentration_array(fermi_energy, h00, h01, disorder_concentration_array, disorder_intensity=2.0, length=100, calculation_times=1, print_show=0):
     dim = np.array(disorder_concentration_array).shape[0]
     conductance_array = np.zeros(dim)
     i0 = 0
-    for disorder_concentration_0 in disorder_concentration_array:
+    for disorder_concentration in disorder_concentration_array:
+        if print_show == 1:
+            print(disorder_concentration)
         for times in range(calculation_times):
-            conductance_array[i0] = conductance_array[i0]+np.real(calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensity=disorder_intensity, disorder_concentration=disorder_concentration_0, length=length))
+            conductance_array[i0] = conductance_array[i0]+np.real(calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensity=disorder_intensity, disorder_concentration=disorder_concentration, length=length))
         i0 += 1
     conductance_array = conductance_array/calculation_times
     return conductance_array
 
-def calculate_conductance_with_scattering_length_array(fermi_energy, h00, h01, length_array, disorder_intensity=2.0, disorder_concentration=1.0, calculation_times=1):
+def calculate_conductance_with_scattering_length_array(fermi_energy, h00, h01, length_array, disorder_intensity=2.0, disorder_concentration=1.0, calculation_times=1, print_show=0):
     dim = np.array(length_array).shape[0]
     conductance_array = np.zeros(dim)
     i0 = 0
-    for length_0 in length_array:
+    for length in length_array:
+        if print_show == 1:
+            print(length)
         for times in range(calculation_times):
-            conductance_array[i0] = conductance_array[i0]+np.real(calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensity=disorder_intensity, disorder_concentration=disorder_concentration, length=length_0))
+            conductance_array[i0] = conductance_array[i0]+np.real(calculate_conductance_with_disorder(fermi_energy, h00, h01, disorder_intensity=disorder_intensity, disorder_concentration=disorder_concentration, length=length))
         i0 += 1
     conductance_array = conductance_array/calculation_times
     return conductance_array
 
+## multi-terminal transmission
+
+def get_gamma_array_and_green_for_six_terminal_transmission(fermi_energy, h00_for_lead_4, h01_for_lead_4, h00_for_lead_2, h01_for_lead_2, center_hamiltonian, width=10, length=50, internal_degree=1, moving_step_of_leads=10):
+    #   ---------------- Geometry ----------------
+    #               lead2         lead3
+    #   lead1(L)                          lead4(R)  
+    #               lead6         lead5 
+
+    # h00 and h01 in leads
+    h00_for_lead_1 = h00_for_lead_4
+    h00_for_lead_2 = h00_for_lead_2
+    h00_for_lead_3 = h00_for_lead_2
+    h00_for_lead_5 = h00_for_lead_2
+    h00_for_lead_6 = h00_for_lead_2
+    h00_for_lead_4 = h00_for_lead_4
+    h01_for_lead_1 = h01_for_lead_4.transpose().conj()
+    h01_for_lead_2 = h01_for_lead_2
+    h01_for_lead_3 = h01_for_lead_2
+    h01_for_lead_4 = h01_for_lead_4
+    h01_for_lead_5 = h01_for_lead_2.transpose().conj()
+    h01_for_lead_6 = h01_for_lead_2.transpose().conj()
+    # hopping matrix from lead to center
+    h_lead1_to_center = np.zeros((internal_degree*width, internal_degree*width*length), dtype=complex)
+    h_lead2_to_center = np.zeros((internal_degree*width, internal_degree*width*length), dtype=complex)
+    h_lead3_to_center = np.zeros((internal_degree*width, internal_degree*width*length), dtype=complex)
+    h_lead4_to_center = np.zeros((internal_degree*width, internal_degree*width*length), dtype=complex)
+    h_lead5_to_center = np.zeros((internal_degree*width, internal_degree*width*length), dtype=complex)
+    h_lead6_to_center = np.zeros((internal_degree*width, internal_degree*width*length), dtype=complex)
+    move = moving_step_of_leads # the step of leads 2,3,6,5 moving to center
+    h_lead1_to_center[0:internal_degree*width, 0:internal_degree*width] = h01_for_lead_1.transpose().conj()
+    h_lead4_to_center[0:internal_degree*width, internal_degree*width*(length-1):internal_degree*width*length] = h01_for_lead_4.transpose().conj()
+    for i0 in range(width):
+        begin_index = internal_degree*i0+0
+        end_index = internal_degree*i0+internal_degree
+        h_lead2_to_center[begin_index:end_index, internal_degree*(width*(move+i0)+(width-1))+0:internal_degree*(width*(move+i0)+(width-1))+internal_degree] = h01_for_lead_2.transpose().conj()[begin_index:end_index, begin_index:end_index]
+        h_lead3_to_center[begin_index:end_index, internal_degree*(width*(length-move-1-i0)+(width-1))+0:internal_degree*(width*(length-move-1-i0)+(width-1))+internal_degree] = h01_for_lead_3.transpose().conj()[begin_index:end_index, begin_index:end_index]
+        h_lead5_to_center[begin_index:end_index, internal_degree*(width*(length-move-1-i0)+0)+0:internal_degree*(width*(length-move-1-i0)+0)+internal_degree] = h01_for_lead_5.transpose().conj()[begin_index:end_index, begin_index:end_index]
+        h_lead6_to_center[begin_index:end_index, internal_degree*(width*(i0+move)+0)+0:internal_degree*(width*(i0+move)+0)+internal_degree] = h01_for_lead_6.transpose().conj()[begin_index:end_index, begin_index:end_index]
+    # self energy    
+    self_energy1, gamma1 = guan.self_energy_of_lead_with_h_lead_to_center(fermi_energy, h00_for_lead_1, h01_for_lead_1, h_lead1_to_center)
+    self_energy2, gamma2 = guan.self_energy_of_lead_with_h_lead_to_center(fermi_energy, h00_for_lead_2, h01_for_lead_1, h_lead2_to_center)
+    self_energy3, gamma3 = guan.self_energy_of_lead_with_h_lead_to_center(fermi_energy, h00_for_lead_3, h01_for_lead_1, h_lead3_to_center)
+    self_energy4, gamma4 = guan.self_energy_of_lead_with_h_lead_to_center(fermi_energy, h00_for_lead_4, h01_for_lead_1, h_lead4_to_center)
+    self_energy5, gamma5 = guan.self_energy_of_lead_with_h_lead_to_center(fermi_energy, h00_for_lead_5, h01_for_lead_1, h_lead5_to_center)
+    self_energy6, gamma6 = guan.self_energy_of_lead_with_h_lead_to_center(fermi_energy, h00_for_lead_6, h01_for_lead_1, h_lead6_to_center)
+    gamma_array = [gamma1, gamma2, gamma3, gamma4, gamma5, gamma6]
+    # Green function
+    green = np.linalg.inv(fermi_energy*np.eye(internal_degree*width*length)-center_hamiltonian-self_energy1-self_energy2-self_energy3-self_energy4-self_energy5-self_energy6)
+    return gamma_array, green
+
+def calculate_six_terminal_transmission_matrix(fermi_energy, h00_for_lead_4, h01_for_lead_4, h00_for_lead_2, h01_for_lead_2, center_hamiltonian, width=10, length=50, internal_degree=1, moving_step_of_leads=10):
+    gamma_array, green = guan.get_gamma_array_and_green_for_six_terminal_transmission(fermi_energy, h00_for_lead_4, h01_for_lead_4, h00_for_lead_2, h01_for_lead_2, center_hamiltonian, width, length, internal_degree, moving_step_of_leads)
+    # Transmission
+    transmission_matrix = np.zeros((6, 6), dtype=complex)
+    channel_lead_4 = guan.calculate_conductance(fermi_energy, h00_for_lead_4, h01_for_lead_4, length=3)
+    channel_lead_2 = guan.calculate_conductance(fermi_energy, h00_for_lead_2, h01_for_lead_2, length=3)
+    for i0 in range(6):
+        for j0 in range(6):
+            if j0!=i0:
+                transmission_matrix[i0, j0] = np.trace(np.dot(np.dot(np.dot(gamma_array[i0], green), gamma_array[j0]), green.transpose().conj()))
+    for i0 in range(6):
+        if i0 == 0 or i0 == 3:
+            transmission_matrix[i0, i0] = channel_lead_4
+        else:
+            transmission_matrix[i0, i0] = channel_lead_2
+    for i0 in range(6):
+        for j0 in range(6):
+            if j0!=i0:
+                transmission_matrix[i0, i0] = transmission_matrix[i0, i0]-transmission_matrix[i0, j0]
+    transmission_matrix = np.real(transmission_matrix)
+    return transmission_matrix
+
+def calculate_six_terminal_transmissions_from_lead_1(fermi_energy, h00_for_lead_4, h01_for_lead_4, h00_for_lead_2, h01_for_lead_2, center_hamiltonian, width=10, length=50, internal_degree=1, moving_step_of_leads=10):
+    gamma_array, green = guan.get_gamma_array_and_green_for_six_terminal_transmission(fermi_energy, h00_for_lead_4, h01_for_lead_4, h00_for_lead_2, h01_for_lead_2, center_hamiltonian, width, length, internal_degree, moving_step_of_leads)
+    # Transmission
+    transmission_12 = np.real(np.trace(np.dot(np.dot(np.dot(gamma_array[0], green), gamma_array[1]), green.transpose().conj())))
+    transmission_13 = np.real(np.trace(np.dot(np.dot(np.dot(gamma_array[0], green), gamma_array[2]), green.transpose().conj())))
+    transmission_14 = np.real(np.trace(np.dot(np.dot(np.dot(gamma_array[0], green), gamma_array[3]), green.transpose().conj())))
+    transmission_15 = np.real(np.trace(np.dot(np.dot(np.dot(gamma_array[0], green), gamma_array[4]), green.transpose().conj())))
+    transmission_16 = np.real(np.trace(np.dot(np.dot(np.dot(gamma_array[0], green), gamma_array[5]), green.transpose().conj())))
+    return transmission_12, transmission_13, transmission_14, transmission_15, transmission_16
 
 ## scattering matrix
 
@@ -1051,13 +1150,13 @@ def calculate_scattering_matrix(fermi_energy, h00, h01, length=100):
             print('Error Alert: scattering matrix is not normalized!')
     return transmission_matrix, reflection_matrix, k_right, k_left, velocity_right, velocity_left, ind_right_active
 
-def print_or_write_scattering_matrix(fermi_energy, h00, h01, length=100, on_print=1, on_write=0):
+def print_or_write_scattering_matrix(fermi_energy, h00, h01, length=100, print_show=1, write_file=0, filename='a', format='txt'):
     if np.array(h00).shape==():
         dim = 1
     else:
         dim = np.array(h00).shape[0]
     transmission_matrix, reflection_matrix, k_right, k_left, velocity_right, velocity_left, ind_right_active = calculate_scattering_matrix(fermi_energy, h00, h01, length)
-    if on_print == 1:
+    if print_show == 1:
         print('\nActive channel (left or right) = ', ind_right_active)
         print('Evanescent channel (left or right) = ', dim-ind_right_active, '\n')
         print('K of right-moving active channels:\n', np.real(k_right[0:ind_right_active]))
@@ -1070,8 +1169,8 @@ def print_or_write_scattering_matrix(fermi_energy, h00, h01, length=100, on_prin
         print('Total reflection of channels:\n',np.sum(np.square(np.abs(reflection_matrix[0:ind_right_active, 0:ind_right_active])), axis=0))
         print('Sum of transmission and reflection of channels:\n', np.sum(np.square(np.abs(transmission_matrix[0:ind_right_active, 0:ind_right_active])), axis=0) + np.sum(np.square(np.abs(reflection_matrix[0:ind_right_active, 0:ind_right_active])), axis=0))
         print('Total conductance = ', np.sum(np.square(np.abs(transmission_matrix[0:ind_right_active, 0:ind_right_active]))), '\n')
-    if on_write == 1:
-        with open('a.txt', 'w') as f:
+    if write_file == 1:
+        with open(filename+'.'+format, 'w') as f:
             f.write('Active channel (left or right) = ' + str(ind_right_active) + '\n')
             f.write('Evanescent channel (left or right) = ' + str(dim - ind_right_active) + '\n\n')
             f.write('Channel               K                                     Velocity\n')
@@ -1110,7 +1209,7 @@ def print_or_write_scattering_matrix(fermi_energy, h00, h01, length=100, on_prin
 
 # Module 9: topological invariant
 
-def calculate_chern_number_for_square_lattice(hamiltonian_function, precision=100):
+def calculate_chern_number_for_square_lattice(hamiltonian_function, precision=100, print_show=0):
     if np.array(hamiltonian_function(0, 0)).shape==():
         dim = 1
     else:
@@ -1118,6 +1217,8 @@ def calculate_chern_number_for_square_lattice(hamiltonian_function, precision=10
     delta = 2*pi/precision
     chern_number = np.zeros(dim, dtype=complex)
     for kx in np.arange(-pi, pi, delta):
+        if print_show == 1:
+            print(kx)
         for ky in np.arange(-pi, pi, delta):
             H = hamiltonian_function(kx, ky)
             vector = guan.calculate_eigenvector(H)
@@ -1141,10 +1242,12 @@ def calculate_chern_number_for_square_lattice(hamiltonian_function, precision=10
     chern_number = chern_number/(2*pi*1j)
     return chern_number
 
-def calculate_chern_number_for_square_lattice_with_Wilson_loop(hamiltonian_function, precision_of_plaquettes=10, precision_of_Wilson_loop=100):
+def calculate_chern_number_for_square_lattice_with_Wilson_loop(hamiltonian_function, precision_of_plaquettes=10, precision_of_Wilson_loop=100, print_show=0):
     delta = 2*pi/precision_of_plaquettes
     chern_number = 0
     for kx in np.arange(-pi, pi, delta):
+        if print_show == 1:
+            print(kx)
         for ky in np.arange(-pi, pi, delta):
             vector_array = []
             # line_1
@@ -1180,7 +1283,7 @@ def calculate_chern_number_for_square_lattice_with_Wilson_loop(hamiltonian_funct
     chern_number = chern_number/(2*pi)
     return chern_number
 
-def calculate_chern_number_for_honeycomb_lattice(hamiltonian_function, a=1, precision=300):
+def calculate_chern_number_for_honeycomb_lattice(hamiltonian_function, a=1, precision=300, print_show=0):
     if np.array(hamiltonian_function(0, 0)).shape==():
         dim = 1
     else:
@@ -1192,6 +1295,8 @@ def calculate_chern_number_for_honeycomb_lattice(hamiltonian_function, a=1, prec
     delta1 = 2*L1/precision
     delta3 = 2*L3/precision
     for kx in np.arange(-L1, L1, delta1):
+        if print_show == 1:
+            print(kx)
         for ky in np.arange(-L3, L3, delta3):
             if (-L2<=kx<=L2) or (kx>L2 and -(L1-kx)*tan(pi/3)<=ky<=(L1-kx)*tan(pi/3)) or (kx<-L2 and  -(kx-(-L1))*tan(pi/3)<=ky<=(kx-(-L1))*tan(pi/3)):
                 H = hamiltonian_function(kx, ky)
@@ -1216,11 +1321,13 @@ def calculate_chern_number_for_honeycomb_lattice(hamiltonian_function, a=1, prec
     chern_number = chern_number/(2*pi*1j)
     return chern_number
 
-def calculate_wilson_loop(hamiltonian_function, k_min=-pi, k_max=pi, precision=100):
+def calculate_wilson_loop(hamiltonian_function, k_min=-pi, k_max=pi, precision=100, print_show=0):
     k_array = np.linspace(k_min, k_max, precision)
     dim = np.array(hamiltonian_function(0)).shape[0]
     wilson_loop_array = np.ones(dim, dtype=complex)
     for i in range(dim):
+        if print_show == 1:
+            print(i)
         eigenvector_array = []
         for k in k_array:
             eigenvector  = guan.calculate_eigenvector(hamiltonian_function(k))  
